@@ -10,7 +10,7 @@
 
 
 name = 'pygraphviz'
-version = '0.34'
+version = '0.35'
 description = "Python interface to Graphviz"
 long_description = """\
 A Python wrapper for the Graphviz Agraph data structure.
@@ -25,7 +25,7 @@ authors = {'Hagberg' : ('Aric Hagberg','hagberg@lanl.gov'),
            'Renieris' : ('Manos Renieris','')
            }
 url = 'http://networkx.lanl.gov/pygraphviz'
-download_url="http://networkx.lanl.gov/download"
+download_url="http://networkx.lanl.gov/wiki/download"
 platforms = ['Linux','Mac OSX','Windows XP/2000/NT']
 keywords = ['Networks', 'Graph Visualization', 'network', 'graph', 'graph drawing']
 classifiers = [
